package Logic
{
	import World.Character;
	import World.World;
	
	import org.flixel.FlxG;
	import org.flixel.FlxObject;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;

	public class CharacterController extends Controller
	{
		public var _character:Character = null;
		public var _behaviour:String = "";
		public var _scared_of_light:Boolean = false;
		public var _scared_of_yells:Boolean = false;
		public var _confort_distance:Number = 50;
		public var _vision_distance:Number = 500;
		public var _last_known_pos:FlxPoint = null;
		
		public function CharacterController()
		{
			super(null);			
		}
		
		public function setCharacter(v:Character):void
		{
			_character = v;
			_target = v;
			_type = "character";
			_character._controller = this;
			_confort_distance += Math.random() * 40 - 20;
		}
		
		public override function update():void
		{
			if(!_character) return;
			
			if(_character == World.World._player && !_character._is_blocked)
				checkPlayerInput();
			else
				computeIA();
		}
		
		public function computeIA():void
		{
			var _player:Character = World.World._player;
			var dist:Number = Math.abs( _player.x - _character.x);
			if(_player._in_map != _character._in_map)
				dist = 100000;
			
			var is_facing:Boolean = false;
			if(_player.facing == FlxObject.RIGHT && _player.x < _character.x)
				is_facing = true;
			if(_player.facing == FlxObject.LEFT && _player.x > _character.x)
				is_facing = true;
			if(dist > 10000)
				is_facing = false;

			var can_see_player:Boolean = false;
			if(dist < _vision_distance)
				can_see_player = true;
			
			var is_near:Boolean = false;
			if(dist < _confort_distance)
				is_near = true;
			
			if(is_near)
			{
				_player.giveLove(0.02 * FlxG.elapsed);
			}
			
			if(_behaviour == "follower")
			{
				if(can_see_player)
					_last_known_pos = new FlxPoint( _player.x, _player.y );
				
				if( dist > _confort_distance && can_see_player)
				{
					if(_player.x > _character.x)
						_character.moveRight();
					else if(_player.x < _character.x)
						_character.moveLeft();
				}
				
				if(!can_see_player && _last_known_pos)
				{
					//tracing("searching");
					if(_last_known_pos.x > _character.x)
						_character.moveRight();
					else if(_last_known_pos.x < _character.x)
						_character.moveLeft();
				}
				else
					_last_known_pos = null;	
			}
			else if(_behaviour == "stalker")
			{
				if( dist > _confort_distance && can_see_player)
				{
					_last_known_pos = new FlxPoint( _player.x, _player.y );
					
					if(!is_facing)
					{
						if(_player.x > _character.x)
							_character.moveRight();
						else if(_player.x < _character.x)
							_character.moveLeft();
					}
					else
					{
						_character.facing = (_player.facing == FlxObject.RIGHT ? FlxObject.RIGHT : FlxObject.LEFT);
						_character.idle();
					}
				}
				
				if(!can_see_player && _last_known_pos)
				{
					//tracing("searching");
					if(_last_known_pos.x > _character.x)
						_character.moveRight();
					else if(_last_known_pos.x < _character.x)
						_character.moveLeft();
				}
				else
					_last_known_pos = null;	
				
			}
			else if(_behaviour == "wanderer")
			{
				if(_character.facing == FlxObject.RIGHT)
					_character.moveRight();
				else
					_character.moveLeft();
				
				//turn sometimes
				if(Math.random() * 100 < 2)
					_character.facing = (_character.facing == FlxObject.RIGHT ? FlxObject.LEFT : FlxObject.RIGHT);
					
			}
			
			var scared:Boolean = false;
			if(_scared_of_light && _player._has_flashlight && is_facing && dist < 150)
				scared = true;
			
			if(_scared_of_yells && _player._is_yelling && dist < 150 && is_facing)
				scared = true;
			
			if(scared)
			{
				_character.runSpeed();
				if(_player.x > _character.x)
					_character.moveLeft();
				else
					_character.moveRight();
			}	
			else
				_character.walkSpeed();
		}
		
		public function checkPlayerInput():void
		{
			if (FlxG.keys.LEFT)
			{
				_character.moveLeft();
			}
			else if (FlxG.keys.RIGHT)
			{
				_character.moveRight();
			}
			
			if (FlxG.keys.justPressed("UP") && _character._entity_behind != null)
			{
				_character.play("back");
				_character._is_idle = true;
				
				if(_character._entity_behind.type == "trigger" && _character._entity_behind.on == "up")
				{
					trace("Action: " + _character._entity_behind.action);
					_character.doAction( _character._entity_behind.action, _character._entity_behind);
				}
				else if(_character._entity_behind.type == "item")
				{
					_character.takeItem();
				}
			}
			
			if (FlxG.keys.justPressed("DOWN") )
			{
				_character.yell();
			}
			
			if (FlxG.keys.justPressed("K") )
			{
				_character.die();
			}

			if (FlxG.keys.justPressed("SPACE") )
			{
				_character.toggleFlashlight();
			}
			
			//jumping
			/*
			if (FlxG.keys.justPressed("SPACE") && _character._is_over_ground)
			{
				_character.jump();
			}
			*/
		}
	}
}