package  
{
	import flash.events.TimerEvent;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.utils.Timer;
	import flash.net.URLRequest;
	import flash.media.SoundTransform;
	
	/**
	 * ...
	 * @author Graham Coleman
	 */
	public class StreetSongs 
	{
		static public var instance:StreetSongs;
		var soundClip:Sound;
		
		var lastGridX = 0;
		var lastGridY = 0;
		var lastLoc = 0;
		
		var sectionBlockSize = 40000 / 2.166666666667; //block at which sections repeat
		var sectionLength = 32000 / 2.166666666667; //length of content
		var hangLength = 8000 / 2.166666666667; //here is the break
		
		var channelPlaying:SoundChannel;
		var channelPrev:SoundChannel;
		var channelHang:SoundChannel;
		
		public var isPrimaryTrans:Boolean = false; //primary transition in progress
		var transTicks:Number = 20;
		var globalTimer:Timer;
		
		public function StreetSongs() 
		{
			instance = this;
			
			soundClip = new Sound();
			soundClip.load(new URLRequest(gamejam.ASSETS_ROOT + "100streets.mp3"));
			soundClip.addEventListener(Event.COMPLETE, soundLoaded);
			
			//global timing
			globalTimer = new Timer(sectionLength, 0); //infinite timer for reset melody
			globalTimer.addEventListener(TimerEvent.TIMER, resetMelody);
		}
		
		//you can put in -1 for transition to silence
		public function transState( targetX:int, targetY:int ) {
			if ( isPrimaryTrans ) return;
			
			isPrimaryTrans = true; //start a primary transition
			
			var sec:Number = (targetY * 5 + targetX + 1)-1;
			var loc:Number = sec * sectionBlockSize;
			
			lastGridX = targetX;
			lastGridY = targetY;
			lastLoc = loc;
	
			//assume it is playing?
			var relPos:Number = (channelPlaying != null) ? channelPlaying.position : 0;
			relPos = relPos % sectionBlockSize;
				
			channelPrev = channelPlaying; //swap prev
				
			if ( targetX != -1 ) {
				var silence:SoundTransform = new SoundTransform(0);
				channelPlaying = soundClip.play(loc + relPos, 0, silence);
				channelPlaying.soundTransform.volume = 0;
				//channelPrev.soundTransform.volume = 1; //redundant
			} else {
				channelPlaying = null;
			}
				
			//set initial volumes
			var timer:Timer = new Timer(100, transTicks);
			timer.addEventListener(TimerEvent.TIMER, transVol);
			timer.addEventListener(TimerEvent.TIMER_COMPLETE, transDone);
			timer.start();
			
		} //end of transState
		
		function transVol(te:TimerEvent) {
			
			if ( channelPlaying != null ) {
				var xform:SoundTransform = channelPlaying.soundTransform;
				xform.volume = te.target.currentCount / transTicks;
				channelPlaying.soundTransform = xform;
			}
			
			if ( channelPrev != null ) {
				xform = channelPrev.soundTransform; //holy crap! having to set this is insane
				xform.volume = (transTicks - te.target.currentCount) / transTicks;
				channelPrev.soundTransform = xform;
			}
		}
		
		function transDone(te:TimerEvent) {
			if ( channelPrev != null ) channelPrev.stop();
			isPrimaryTrans = false;
			//trace('finished trans');
		}
		
		function resetMelody(te:TimerEvent) {
			//strategy: switch the channelPlaying reference
			channelHang = channelPlaying;
			channelPlaying = soundClip.play(lastLoc, 0, null);
			
			var hangTimer:Timer = new Timer( hangLength, 1 );
			hangTimer.addEventListener(TimerEvent.TIMER, hangStop);
			hangTimer.start();
		}
		
		function hangStop(te:TimerEvent) {
			if ( channelHang != null ) {
				channelHang.stop();
			}
		}
		
		function soundLoaded(e:Event) {
			//channelPlaying = soundClip.play();
			globalTimer.start();
		}
		
		private function errorHandler(errorEvent:IOErrorEvent):void {
            trace("The sound could not be loaded: " + errorEvent.text);
        }
		
	}

}