package World
{
	import Logic.CharacterController;
	
	import Utils.Generic;
	
	import flash.display.*;
	import flash.events.*;
	import flash.events.TimerEvent;
	import flash.utils.*;
	import flash.xml.*;
	
	import org.flixel.*;

	public class Character extends FlxExtSprite
	{
		//profile
		public var _street:Street = null;
		public var _info:Object = {};
		public var _controller:CharacterController = null;
		
		//state
		public var _is_over_ground:Boolean = false;
		public var _inactivity_time:uint = 0;
		public var _entity_behind:Object = null;
		public var _is_idle:Boolean = true;
		public var _is_blocked:Boolean = false;
		public var _has_flashlight:Boolean = false;
		public var _in_map:Map = null;
		public var _is_running:Boolean = false;
		public var _is_yelling:Boolean = false;
		public var _mood:String = "normal";
		
		public var _items:Array = [];
		
		//network sync
		public var _last_msg_timestamp:Number = 0;
		public var _last_msg:String = "";
		public var _last_serialize:String = "";
		
		//const
		
		//icons
		[Embed(source='assets/imgs/icons/icon_arrow.png')] private var ImgArrow:Class;		
		[Embed(source='assets/imgs/icons/icon_hand.png')] private var ImgHand:Class;		
		public var _current_icon:FlxSprite = null;
		public var _current_icon_name:String = null;
		public var _flashlight:FlxExtSprite = null;

		public var RUNNING_SPEED:Number = 60;
		
		//events
				
		//ctors and init
		public function Character(X:Number=0, Y:Number=0, SimpleGraphic:Class=null)
		{
			super(X, Y, SimpleGraphic);
			
			acceleration.y = World.World.GRAVITY; 
			drag.x = 100;
			health = 100;
			facing = RIGHT;
			//scale.x = scale.y = 2;
			maxVelocity.x = World.World.MAXVELX;
			maxVelocity.y = World.World.MAXVELY;			
		}
		
		//actions
		public function doAction(name:String, data:Object):void
		{
			if(name == "changemap" && !World.Map._switching_map && (getTimer() - World.Map._last_map_switch) > 1000)
			{
				var a:Array = (data.extra as String).split(":");
				teleportTo( a[0],0,0, ( a.length > 1 ? a[1] : null ) );
			}
			else if(name == "give")
			{
				if(data && data.extra == "flashlight")
					this.setFlashlight(true);				
			}
			else if(name == "showmsg")
			{
				idle();
				play("back");
				PlayState.showMsg(data.extra,10);
			}
			else if(name == "supplies")
			{
				PlayState.showMsg("I've found some supplies!",10);
				giveSupplies( parseFloat(data.extra) );
			}
		}
		
		public function giveSupplies(v:Number):void
		{
			_info["supplies"] += v;
			if( _info["supplies"] > 1)
				_info["supplies"] = 1;
			
			PlayState._inst.updateBars();
		}
		
		public function giveLove(v:Number):void
		{
			_info["loneliness"] += v;
			if( _info["loneliness"] > 1)
				_info["loneliness"] = 1;
			
			PlayState._inst.updateBars();
		}
		
		public function teleportTo(map_id:String, x:uint, y:uint, target_object_id:String = null, on_complete:Function = null):void
		{
			var map:Map = World.World.getMap(map_id);
			if (!map)
			{
				trace("map not found: " + map_id);
				return;
			}
			blockInteraction(true);
			
			var that:Character = this;
			if(World.World._player == this) //if player, load new map
				map.switchTo(inner);
			else
				inner();
			
			function inner():void
			{
				that.setMap(map_id);
				if(target_object_id)
					that.moveTo(target_object_id);
				else
					that.moveTo("start");
				that.x += x;
				that.y += y;
				that.velocity.x = 0;
				that.velocity.y = 0;
				that.play("idle");
				that.blockInteraction(false);
				
				if(on_complete != null)
					on_complete();
			}
		}

		public function idle():void
		{
			velocity.x = 0;
			_inactivity_time = 0;
			_is_idle = true;
			if(_has_flashlight)
				play("idle_flashlight");
			else
				play("idle");			
		}
		
		public function moveLeft():void
		{
			facing = FlxObject.LEFT;
			velocity.x -= World.World.WALK_SPEED * FlxG.elapsed;
			_inactivity_time = 0;
			_is_idle = false;
		}
		
		public function moveRight():void
		{
			facing = FlxObject.RIGHT;
			velocity.x += World.World.WALK_SPEED * FlxG.elapsed;
			_inactivity_time = 0;
			_is_idle = false;
		}
		
		public function jump():void
		{
			velocity.y = -World.World.JUMP_POWER;
			y -= 1;
			_inactivity_time = 0;
			_is_idle = false;			
		}

		public function runSpeed():void
		{
			var speed:Number = World.World.MAXVELX;
			if(_info["speed"])
				speed *= parseFloat(_info["speed"]);
			
			maxVelocity.x = speed * 2;
		}		
		
		public function walkSpeed():void
		{
			var speed:Number = World.World.MAXVELX;
			if(_info["speed"])
				speed *= parseFloat(_info["speed"]);
			
			maxVelocity.x = speed;
		}		
		
		public function takeItem():void
		{
			if(_is_blocked) return; //non interactive moment
			if(!_entity_behind || _entity_behind.type != "item") return;
			trace("Taking item:" + _entity_behind.subtype);
			
			var item:Item = _entity_behind.sp as Item;
			if(!item) return;

			blockInteraction(true);
			play("back");
			var that:Character = this;
			var myTimer:Timer = new Timer(200, 1);
			myTimer.addEventListener(TimerEvent.TIMER_COMPLETE, inner);
			myTimer.start();
			
			function inner(e:TimerEvent):void
			{
				item.take();
				that.giveSupplies(0.2);
				
				that._items.push(_entity_behind);
				that._entity_behind = null;
				that.blockInteraction(false);
				that.idle();
			}
		}
		
		public function yell():void
		{
			play("yell");	
			_is_blocked = true;
			_is_yelling = true;
			setFlashlight(false);
			
			var that:Character = this;
			
			var myTimer:Timer = new Timer(800, 1);
			myTimer.addEventListener(TimerEvent.TIMER_COMPLETE, inner);
			myTimer.start();
			
			function inner(e:TimerEvent):void
			{
				that._is_blocked = false;
				that._is_yelling = false;
				//that.play("idle");
			}
			
		}
		
		public function die():void
		{
			if(health == 0) return; //already dead
			
			play("die");			
			_is_blocked = true;
			setFlashlight(false);
			health = 0;
			_info["loneliness"] = 0;
			_info["supplies"] = 0;

			/*
			var that:Character = this;
			
			var myTimer:Timer = new Timer(2000, 1);
			myTimer.addEventListener(TimerEvent.TIMER_COMPLETE, inner);
			myTimer.start();
			
			function inner(e:TimerEvent):void
			{
				that.play("dead");
			}
			*/
		}
		
		public function setMap(map_id:String):void
		{
			_in_map = World.World.getMap(map_id);
		}
		

		public function setFlashlight(v:Boolean):void
		{
			if(v == _has_flashlight) return;
			
			_is_idle = false;
			if(!v)
			{
				trace("switching OFF flashlight");
				_has_flashlight = false;
				if(_flashlight)
					World.World._fg_group.remove(_flashlight);
				_flashlight = null;
				return;
			}
			
			trace("switching ON flashlight");
			_has_flashlight = true;
			
			var item:Object = Item.getItemType("flashlight");
			if(!item) return;
			
			var bitmap:Bitmap = ResourcesLoader.getFile(item.src);
			if (bitmap == null)
				return;
			
			_flashlight = new FlxExtSprite(0,0);
			_flashlight.loadExtGraphic( bitmap,false,true,0,0,true);
			_flashlight.blend = "add";
			World.World._fg_group.add(_flashlight);
		}		
		
		public function toggleFlashlight(v:Boolean = false):void
		{
			setFlashlight(!_has_flashlight);
		}
		
		public function setIcon(name:String):void
		{
			if(_current_icon_name == name) return;
			
			if(_current_icon_name)
			{
				World.World._fg_group.remove(_current_icon);
				_current_icon = null;
			}
			
			_current_icon_name = name;
			trace("setting icon");
			
			var icon:FlxSprite = new FlxSprite(0,0);
			
			if(name == "arrow")
				icon.loadGraphic(ImgArrow,true,true,15,15);
			else if(name == "grab")
				icon.loadGraphic(ImgHand,true,true,15,15);
			else
			{
				if(_current_icon)
				{
					World.World._fg_group.remove(_current_icon);
					_current_icon = null;
				}
				return;
			}
			
			icon.addAnimation("idle",[0,1],2);
			icon.play("idle");
			
			_current_icon = icon;
			World.World._fg_group.add(_current_icon);
		}
		
		public function moveTo(object_id:String):void
		{
			var ent:Object = _in_map.getEntity(object_id);
			if(!ent) return;
			x = ent.x;
			y = ent.y;
		}
		
		public function blockInteraction(v:Boolean):void
		{
			_is_blocked = v;
			velocity.x = 0;
			velocity.y = 0;
		}
		
		//state
		public function show():void
		{
			
		}
		
		public function hide():void
		{
			
		}
		
		//info
		public function isPlayer():Boolean
		{
			return this == World.World._player;
		}
		
		//update
        override public function update():void
        {
			//visible
			visible = (_in_map == World.World._map);
			
			//ground collision
			var off:int = (-this.height + this.offset.y);	
			if (y - off > World.World.GROUND_HEIGHT)
			{
				y = World.World.GROUND_HEIGHT + off;
				velocity.y = 0;  
				_is_over_ground = true;
			}
			else
				_is_over_ground = false;
			
			if (!_is_blocked)
			{
				if (velocity.x == 0 && velocity.y == 0)
				{
					if(!_is_idle)
					{
						idle();
					}
					
					if(_is_idle && !_has_flashlight && _inactivity_time > 300)
					{
						play("wait");
						_inactivity_time = 0;
					}
				}
				else 
				{ 
					if (velocity.y == 0)
					{
						if(_has_flashlight)
							play("walk_flashlight");
						else if( Math.abs(velocity.x) < RUNNING_SPEED)
							play("walk");
						else
							play("run");

					}
					else
						play("jump");
				}		
			}//acting
			
			_inactivity_time += 1;
			
			if(isPlayer() && health > 0)
			{
				_info["loneliness"] -= FlxG.elapsed * 0.01;
				if( _info["loneliness"] <= 0 )
					die();
				
				if(!_is_idle)
					_info["supplies"] -= FlxG.elapsed * 0.01;
				if( _info["supplies"] <= 0 )
					die();				
			}
			
			super.update();
		}

		public override function postUpdate():void
		{
			super.postUpdate();

			if(_current_icon)
			{
				_current_icon.x = x+7;
				_current_icon.y = y-_current_icon.height+5;
			}
			
			if(_flashlight)
			{
				if(_facing == RIGHT)
					_flashlight.x = x+27;
				else
					_flashlight.x = x-_flashlight.width;
				_flashlight.facing = _facing;
				
				_flashlight.y = y+27;	
				
				_flashlight.visible = !_is_blocked;
			}
			
		}
		
		public function configure(character_id:String):Boolean
		{
			//find char type
			var o:Object = World.World._character_types[character_id];
			if(o == null)
			{
				trace("Character type not found: " + character_id);
				return false;
			}
			
			//chose image random
			var img:Object = null;
			img = o.imgs[ Math.floor((Math.random() * o.imgs.length))];
			
			var bitmap:Bitmap = null;
			bitmap = ResourcesLoader.getFile( img.src );
			if (bitmap == null)
			{
				trace("bitmap not loaded: " + img.src );
				return false;
			}
			
			loadExtGraphic( bitmap,true,true,img.size.x,img.size.y,true);
			
			for each(var anim:Object in o.anims)
			addAnimation(anim.id, anim.frames, anim.fps, anim.loop);
			play("idle",true);
			
			//vars
			for(var v:String in o.vars)
				_info[v] = o.vars[v];
			
			if(_info["speed"])
				maxVelocity.x = parseFloat(_info["speed"]) * World.World.MAXVELX;
			maxVelocity.x += Math.random() * 6 - 3;
			
			return true;
		}
				
		
		static public function addTypeFromXML(xml:XMLNode):void
		{
			if (xml.localName != "character")
				return;
			
			var character:Object = {};
			character.id = xml.attributes.id;
			character.imgs = [];
			character.anims = [];
			character.vars = {};
			
			trace("character: " + character.id);				
			
			for each(var subnode:XMLNode in xml.childNodes)
			{
				if (subnode.localName == "img")
				{
					var img:Object = {};
					if(subnode.attributes.id)
						img.id = subnode.attributes.id;
					img.src = gamejam.ASSETS_ROOT + subnode.attributes.src;
					img.size = UtilsFlx.parsePoint(subnode.attributes.size);
					img.offset = UtilsFlx.parsePoint(subnode.attributes.offset);
					character.imgs.push(img);
					
					World.World.includeResource(img.src);
				}
				else if (subnode.localName == "anim")
				{
					var anim:Object = {};
					anim.id = subnode.attributes.id;
					if (subnode.attributes.frames)
						anim.frames = UtilsFlx.parseArray(subnode.attributes.frames);
					if (subnode.attributes.offset)
						anim.offset = UtilsFlx.parsePoint(subnode.attributes.offset);
					else
						anim.offset = [0,0];
					if (subnode.attributes.fps)
						anim.fps = parseInt(subnode.attributes.fps);
					else
						anim.fps = 30;
					if (subnode.attributes.loop)
						anim.loop = (subnode.attributes.loop == "1");
					character.anims.push(anim);
				}
				else if (subnode.localName == "var")
				{
					character.vars[ subnode.attributes.name ] = subnode.attributes.value;
				}
			}
			
			World.World._character_types[character.id] = character;			
		}
		
		
		/**************************/
		
		public function unserialize(str:String):void
		{
			if (str == "") return;
			
			var a:Array = str.split('|');			
			a.shift(); //msg type
			a.shift(); //id
			_street = a.shift();
			x = int(a.shift());
			y = int(a.shift());
			facing = a.shift() == 'r' ? RIGHT : LEFT;
			velocity.x = a.shift();
			velocity.y = a.shift();
			play(a.shift());
			
			if (x == -100 && y == -100)
				World.World.removePerson(_id);
		}
		
		public function serialize():String
		{
			var s:String = "";
			s += _street;
			s += "|" + x.toFixed(3) + "|" + y.toFixed(3);
			s += "|" + ( this.facing == LEFT ? 'l' : 'r' );
			s += "|" + velocity.x.toFixed(3) + "|" + velocity.y.toFixed(3);
			//s += " " + acceleration.x.toFixed(3) + " " + acceleration.y.toFixed(3);
			if (_curAnim)
				s += "|" + _curAnim.name;
			else
				s += "|normal";
			
			
			_last_serialize = s;
			return s;
		}		
		
	}
}