package World
{
	import flash.display.Bitmap;
	import flash.utils.getTimer;
	import flash.xml.XMLNode;
	
	import org.flixel.FlxExtSprite;
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxSprite;
	import org.flixel.FlxU;
	
	public class Item extends FlxExtSprite
	{
		public var _coll:FlxExtSprite = null;
		
		public function Item(X:Number=0, Y:Number=0, SimpleGraphic:Class=null)
		{
			super(X, Y, SimpleGraphic);
		}
		
		static public function getItemType(item_id:String):Object
		{
			return World.World._item_types[item_id];
		}
		
		public function configure(item_id:String):Boolean
		{
			//find char type
			var o:Object = World.World._item_types[item_id];
			if(o == null)
			{
				trace("Item type not found: " + item_id);
				return false;
			}
			
			//chose image random
			var img:Object = null;
			if(o.imgs.length)
				img = o.imgs[ Math.floor((Math.random() * o.imgs.length))];
			else
				img = o.src;
			
			if(img == null)
			{
				trace("no image for item");
				return false;
			}
			
			var bitmap:Bitmap = null;
			bitmap = ResourcesLoader.getFile( img.src );
			if (bitmap == null)
			{
				trace("bitmap not loaded: " + img.src );
				return false;
			}
			
			loadExtGraphic( bitmap,true,true,bitmap.bitmapData.width,bitmap.bitmapData.height,true);
			
			_coll = new FlxExtSprite(0,0);
			_coll._entity = _entity;
			_coll.makeGraphic(bitmap.bitmapData.width,bitmap.bitmapData.height,0xff00ffff);
			
			/*
			for each(var anim:Object in o.anims)
			addAnimation(anim.id, anim.frames, anim.fps);
			play("idle",true);
			*/
			
			return true;
		}
		
		public function drop(layer:FlxGroup = null):void
		{
			World.World._bg_group.add(this);
			World.World._coll_group.add(_coll);
		}
		
		public function take():void
		{
			this.remove();
			
			if(!_entity) return;
			
			if(_entity.count)
				_entity.count -= 1;
			_entity.last_time = getTimer();
		}

		public function remove():void
		{
			World.World._bg_group.remove(this);
			World.World._coll_group.remove(_coll);
		}
		
		public override function update():void
		{
			if(_coll)
			{
				_coll.x = this.x;
				_coll.y = this.y;
			}
		}

		static public function addTypeFromXML(xml:XMLNode):void
		{
			if (xml.localName != "item")
				return;
			
			var item:Object = {};
			item.id = xml.attributes.id;
			if(xml.attributes.src)
			{
				item.src = gamejam.ASSETS_ROOT + xml.attributes.src;
				World.World.includeResource(item.src);
			}
			
			item.imgs = [];
			item.anims = [];

			trace("item: " + item.id);				
			
			for each(var subnode:XMLNode in xml.childNodes)
			{
				if (subnode.localName == "img")
				{
					var img:Object = {};
					if(subnode.attributes.id)
						img.id = subnode.attributes.id;
					img.src = gamejam.ASSETS_ROOT + subnode.attributes.src;
					if(img.size)
						img.size = UtilsFlx.parsePoint(subnode.attributes.size);
					if(img.offset)
						img.offset = UtilsFlx.parsePoint(subnode.attributes.offset);
					item.imgs.push(img);
					
					World.World.includeResource(img.src);
				}
				else if (subnode.localName == "anim")
				{
					var anim:Object = {};
					anim.id = subnode.attributes.id;
					if (subnode.attributes.frames)
						anim.frames = UtilsFlx.parseArray(subnode.attributes.frames);
					if (subnode.attributes.offset)
						anim.offset = UtilsFlx.parsePoint(subnode.attributes.offset);
					else
						anim.offset = [0,0];
					if (subnode.attributes.fps)
						anim.fps = parseInt(subnode.attributes.fps);
					else
						anim.fps = 30;
					item.anims.push(anim);
				}
			}
			
			World.World._item_types[item.id] = item;			
		}
	
	}
}