package World
{
	import Logic.Controller;
	
	import flash.display.Bitmap;
	import flash.utils.getTimer;
	import flash.xml.XMLNode;
	
	import org.flixel.FlxExtSprite;
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	

	public class Map
	{
		//public var _buildings:Array = []
		public var _exits:Array = [];
		public var _items:Array = [];
		
		public var _id:String = "";
		public var _length:uint = 0;
		public var _ground:uint = 100;
		public var _bounds:Array = [0,0,1500,100];
		
		public var _sprites:Array = [];
		public var _entities:Array = []; //configuration entities reprsenting the scene
		public var _entities_by_id:Object = {};
		//public var _res:Object = [];
		
		static public var _switching_map:Boolean = false;
		static public var _last_map_switch:Number = 0;
		
		public function Map()
		{
		}
		
		//change to this map
		public function switchTo(on_complete:Function = null):void
		{
			//if(_switching_map) return;
			trace("Switching to " + _id);

			_switching_map = true;
			World.World._collisions_enabled = false;
			var that:Map = this;
			
			if(World.World._map)
				FlxG.fade(0xff000000,1,false, inner,true); //fadeout
			else
				inner();
						
			
			function inner():void
			{
				trace("fade finishes");
				
				if(World.World._map != that)
				{
					if(World.World._map)
						World.World._map.hide();
				
					World.World._map = that;
					show();	
					//that.updatePeopleVisibility();
				}
				
				FlxG.fade(0xff000000,1,true,null,true);
				
				if(on_complete != null)
					on_complete();
				_switching_map = false;
				World.World._collisions_enabled = true;
				World.Map._last_map_switch = getTimer();
			}
		}
		
		public function getEntity(id:String):Object
		{
			return _entities_by_id[id];
		}
		
		public function getSprite(id:String):FlxSprite
		{
			var r:Object = _entities_by_id[id];
			if(r) return r.sp;
			return null;
		}		
		
		protected function show():void
		{
			trace("Showing map: " + _id);
			World.World.GROUND_HEIGHT = _ground;
			var layer:FlxGroup = World.World._bg_group;
			
			for each(var ent:Object in _entities)
			{
				var bitmap:Bitmap = null;
				var sp:FlxExtSprite = null;
				
				//create FlxSprite to contain it
				
				if(ent.type == "sprite")
				{
					sp = new FlxExtSprite(ent.x, ent.y);
					
					if(ent.id)	sp._id = ent.id;
					ent.sp = sp;
					sp._entity = ent;
					
					if (ent.res)
					{
						bitmap = ResourcesLoader.getFile( ent.res );
						if (bitmap == null)
						{
							trace("bitmap not loaded: " + ent.res );
							continue;
						}
					
						//load image
						if(ent.anim == null)
							sp.loadExtGraphic( bitmap,false,false,0,0,true);
						else //if it is an animated sprite
						{
							sp.loadExtGraphic( bitmap,true,false,ent.anim[0],ent.anim[1],true);
							sp.addAnimation("idle",UtilsFlx.getRangeArray(bitmap.bitmapData.width / ent.anim[0]),ent.anim[2]);
							sp.play("idle",true);
						}
					}
					else
					{
						if(ent.size)
							sp.makeGraphic(ent.size.x,ent.size.y,ent.color,true);
						else
							sp.makeGraphic(2,2,ent.color,true);
					}
					
					if(ent.centered)
					{
						sp.x = ent.x - sp.width * 0.5;
						sp.y = ent.y - sp.height * 0.5;
					}
	
					if (ent.scroll)
						sp.scrollFactor = ent.scroll;
					if (ent.blend)
						sp.blend = ent.blend;
					if (ent.alpha != null)
						sp.alpha = ent.alpha;
					if (ent.vel)
						sp.velocity = ent.vel;
					if (ent.angle)
						sp.angle = ent.angle;
					if (ent.angle_vel)
						sp.angularVelocity = ent.angle_vel;
					if (ent.antialiasing)
						sp.antialiasing = ent.antialiasing;
					if (ent.scale)
						sp.scale = ent.scale;
					if (ent.visible != null)
						sp.visible = ent.visible;
													
					if (ent.controller)
						new Controller(sp, ent.controller);
					
					_sprites.push(sp);
					layer.add(sp);
				}
				else if (ent.type == "item")
				{
					var there_is:Boolean = true;
					
					if(ent.count != null && ent.count == 0)
						there_is = false;
					
					if(ent.delay != null && ent.last_time && (getTimer() - ent.last_time) < ent.delay*1000)
						there_is = false;
					
					if(there_is)
					{
						var item:Item = new Item(ent.x, ent.y);
						item._entity = ent;
						item.configure(ent.subtype);
						
						if(ent.id)	item._id = ent.id;
						ent.sp = item;
						
						item.drop();
						_sprites.push(item);
					}
				}
				else if (ent.type == "layer")
				{
					layer = World.World._fg_group;
				}
				else if (ent.type == "trigger")
				{
					sp = new FlxExtSprite(ent.x, ent.y);
					sp._entity = ent;
					
					if(ent.id)	sp._id = ent.id;					
					ent.sp = sp;
					sp.makeGraphic(ent.size.x,ent.size.y,0xffff00ff,true);
					World.World._coll_group.add(sp);
					_sprites.push(sp);
				}
			}
		}
		
		protected function hide():void
		{
			trace("Hiding map: " + _id);
			
			var layer:FlxGroup = World.World._bg_group;
			
			for each(var ent:Object in _entities)
			{
				if (ent.type == "layer")
					layer = World.World._fg_group;
				else if(ent.type == "trigger" && ent.sp)
				{
					trace("removing trigger");
					World.World._coll_group.remove(ent.sp);
					ent.sp = null;
					continue;
				}
				if (ent.type == "item")
				{
					var item:Item = ent.sp as Item;
					if(item)
						item.remove();
				}
	
				if(ent.sp)
				{
					layer.remove(ent.sp);
					ent.sp = null;
				}
			}		
			
			/*
			for each(var sprite:FlxSprite in _sprites)
			{
				World.World._bg_group.remove(sprite);
			}
			*/
			_sprites.length = 0;
		}
		
		public function updatePeopleVisibility():void
		{
			var map:Map = World.World._map;
			for each(var i:uint in World.World._people)
			{
				var c:Character = World.World._people[i];
				if( c._in_map != map)
					c.visible = false;
				else
					c.visible = true;
			}
		}
		
		public function load(on_complete:Function):void
		{
			//check that all the resources has been loaded
		}
		
		public function fromXML(xml:XMLNode):void
		{
			var node:XMLNode = null;
			
			_id = xml.attributes.id;
			if (xml.attributes.ground)
				_ground = parseInt(xml.attributes.ground);
			
			if (xml.attributes.bounds)
				_bounds = UtilsFlx.parseArray(xml.attributes.bounds);
			
			
			//parse items
			for each (node in xml.childNodes)
			{
				var ent:Object = {type:node.localName};
				
				if (node.attributes.id)
				{
					ent.id = node.attributes.id;
					_entities_by_id[ent.id] = ent;
				}

				//basics
				if (node.attributes.pos)
				{
					ent.pos = node.attributes.pos.split(",");
					ent.x = parseFloat(ent.pos[0]);
					ent.y = parseFloat(ent.pos[1]);
				}
				
				if (node.attributes.scale)
					ent.scale = UtilsFlx.parsePoint( node.attributes.scale );
				if (node.attributes.scroll)
					ent.scroll = UtilsFlx.parsePoint( node.attributes.scroll );
				if (node.attributes.size)
					ent.size = UtilsFlx.parsePoint( node.attributes.size );

				//phisics
				if (node.attributes.vel)
					ent.vel = UtilsFlx.parsePoint(node.attributes.vel);
				if (node.attributes.angle)
					ent.angle = parseFloat(node.attributes.angle);
				if (node.attributes.angle_vel)
					ent.angle_vel = parseFloat(node.attributes.angle_vel);
				
				//visual
				if(node.localName == "sprite")
				{
					ent.blend = node.attributes.blend;
					if (node.attributes.color)
						ent.color = parseInt("0xff" + node.attributes.color);
					else
						ent.color = "0xffffffff";
					
					if (node.attributes.alpha)
						ent.alpha = parseFloat(node.attributes.alpha);
					if (node.attributes.visible)
						ent.visible = (node.attributes.visible == "1");
					if (node.attributes.antialiasing != null)
						ent.antialiasing = node.attributes.antialiasing == "1";
					if (node.attributes.centered != null)
						ent.centered = (node.attributes.centered == "1");

					//resources
					if (node.attributes.src)
					{
						ent.res = gamejam.ASSETS_ROOT + node.attributes.src;
						World.World.includeResource(ent.res);
					}					

					//behaviour
					if (node.attributes.anim)
					{
						var b:Array = node.attributes.anim.split(",");
						ent.anim = b;
					}
				}

				if (node.attributes.controller)
					ent.controller = node.attributes.controller;

				if (node.attributes.type)
					ent.subtype = node.attributes.type;

				if(node.localName == "trigger")
				{
					if (node.attributes.action)
						ent.action = node.attributes.action;					
					if (node.attributes.on)
						ent.on = node.attributes.on;					
					if (node.attributes.icon)
						ent.icon = node.attributes.icon;					
				}

				if(node.localName == "item")
				{
					if (node.attributes.count)
						ent.count = parseInt(node.attributes.count);					
					if (node.attributes.delay)
						ent.delay = parseInt(node.attributes.delay);					
				}
				
				
				if (node.attributes.extra)
					ent.extra = node.attributes.extra;					
									
				//save in container
				_entities.push(ent);
			}//for
		} //func
		
		
	}
}