/***
 * World is the DATABASE for our world, it contains everything
 * and methods to access that data in a safe way.
 * It also dispatch messages according to what happens in the world.
 * WORLD DOES NOT HANDLE GAME LOGIC
 ***/

package World
{
	import Logic.Controller;
	
	import Utils.Generic;
	
	import flash.display.*;
	import flash.events.*;
	import flash.utils.*;
	import flash.xml.*;
	
	import org.flixel.*;
		
	public class World extends EventDispatcher
	{
		static public var _inst:World = null;
		static public var _ST:FlxState = null;
				
		//containers
		static public var _people:Array = []; //instanced people
		static public var _character_types:Dictionary = new Dictionary; //posible characters
		static public var _item_types:Dictionary = new Dictionary; //posible items
		static public var _maps:Dictionary = new Dictionary;
		static public var _resources:ResourcesLoader = new ResourcesLoader;
		static public var _resources_to_load:Object = {};
		
		//main vars
		static public var _player:Character = null;
		static public var _map:Map = null;
		static public var _focus:FlxObject = null;
		static public var _collisions_enabled:Boolean = true;
		
		//groups
		static public var _bg_group:FlxGroup = new FlxGroup();
		static public var _main_group:FlxGroup = new FlxGroup();
		static public var _fg_group:FlxGroup = new FlxGroup();
		static public var _coll_group:FlxGroup = new FlxGroup();
		
		//controllers
		static public var _controllers:Array = [];
		
		//constants
		static public var GRAVITY:int = 400;
		static public var MAXVELX:uint = 50;
		static public var MAXVELY:uint = 100;
		static public var GROUND_HEIGHT:int = 100;
		static public var JUMP_POWER:uint = 200;
		static public var WALK_SPEED:int = 400;
		
		//world config
		static public var start_map:String = "";
		
		//events
		static public var NEW_PERSON:String = "new person";
		
		public function World()
		{
			_inst = this;
		}
		
		public function init(st:FlxState):void
		{
			trace("world init");
			FlxG.worldBounds.x = -50;
			FlxG.worldBounds.width = 1600;
			
			_ST = st;
			
			_bg_group = new FlxGroup();
			_ST.add(_bg_group);
			_main_group = new FlxGroup();
			_ST.add(_main_group);
			_fg_group = new FlxGroup();
			_ST.add(_fg_group);
			
			_coll_group = new FlxGroup();
			_coll_group.visible = false;
			_ST.add(_coll_group);	//show collisions
			
			//_player = new Person();
			_focus = new FlxObject(50,50);
						
			//FlxG.addCamera(_camera);
			FlxG.camera.setBounds(0,0,10,10);
		}
		
		/*** CAMERA ***/
		public function focus(x:int, y:int, tween:Boolean = false):void
		{
			_focus.x = x;
			_focus.y = y;
		}
		
		public function follow(p:Character):void
		{
			
		}
		
		/*** UPDATE ***/
		public function update():void
		{
			updateCamera();
			updateControllers();
			if(_collisions_enabled)
				updateCollisions();
		}
		
		public function updateCollisions():void
		{
			if(!_player) return;
			
			_player._entity_behind = null;
			FlxG.overlap(_player,_coll_group,on_collision);
				
			function on_collision(a:FlxObject, b:FlxObject):void
			{
				if(!World.World._collisions_enabled) return; //stop them
				
				var sp:FlxExtSprite = b as FlxExtSprite;
				if(!sp._entity) return;
				
				if (sp._entity.on == "over" && !_player._is_blocked)
				{
					if(sp._entity.action == "block")
					{
						_player.velocity.x = 0;
						_player.x = _player.last.x;
						_player.idle();
					}
					else
						_player.doAction(sp._entity.action, sp._entity);
				}
				else
				{
					_player._entity_behind = sp._entity;
					
					if(sp._entity.type == "item")
						_player.setIcon("grab");
					else if(sp._entity.type == "trigger")
					{
						if(sp._entity.icon)
							_player.setIcon(sp._entity.icon);
					}
				}
			}
			
			if(!_player._entity_behind)
				_player.setIcon("");
		}
		
		public function updateCamera():void
		{
			FlxG.camera.follow( _focus );
			if(_map)
				FlxG.camera.setBounds(_map._bounds[0],_map._bounds[1],_map._bounds[2],_map._bounds[3]);
				//FlxG.camera.setBounds(0,0,1500,100);

			FlxG.camera.update();
			
			//_camera.x = Math.sin(gamejam.frames) * 100;
			/*
			_camera.update();
			/*
			//if (_player) FlxG.follow(_player,0.01);
			if (_camera) FlxG.follow(_camera,0.01);
			
			FlxG.followAdjust(1.1, 1.1);
			//if (current_street)	FlxG.followBounds(0,0,current_street.length,100);
			*/
		}
		
		public function updateControllers():void
		{
			Controller.updateAll();
		}
		
		/*** CONFIGURE ***/
		public function fromXML(xml:XMLNode):void
		{
			var node:XMLNode = null;
			
			//maps
			for each (node in xml.childNodes)
			{
				if (node.localName == "maps")
				{
					loadMaps(node);
				}
				else if (node.localName == "characters")
				{
					loadCharacters(node);
				}
				else if (node.localName == "items")
				{
					loadItems(node);
				}
				else
				{
					trace("unknown tag");
					
				}
			}
			
			
		}
		
		private function loadMaps(xml:XMLNode):void
		{
			start_map = xml.attributes.start;
			
			for each(var node:XMLNode in xml.childNodes)
			{
				if (node.localName == "map")
				{
					var map:Map = new Map();
					map.fromXML(node);
					addMap(map);
				}
			}
		}
		
		private function loadCharacters(xml:XMLNode):void
		{
			for each(var node:XMLNode in xml.childNodes)
			{
				if (node.localName == "character")
					Character.addTypeFromXML(node);
			}
		}
		
		private function loadItems(xml:XMLNode):void
		{
			for each(var node:XMLNode in xml.childNodes)
			{
				if (node.localName == "item")
					Item.addTypeFromXML(node);
			}
		}
		
		public function loadResources(on_complete:Function=null):void
		{
			var num:int = 0;
			for each(var res:String in _resources_to_load)
			{
				_resources.enqueue(res);
				num += 1;
			}
				
			trace("Loading world resources... " + num);
			_resources.load(on_complete);
		}
		
		static public function includeResource(filename:String):void
		{
			if ( ResourcesLoader.getFile(filename) == null)
			{
				_resources_to_load[filename] = filename;
				trace("Included res: " + filename);
			}
		}
		
		/*** PEOPLE ***/
		static public function addPerson(p:Character):void
		{
			_people.push(p);			
		}
		
		static public function getPerson(id:String):Character
		{
			for each(var i:uint in _people)
				if(_people[i]._id == id)
					return _people[i]; 
			return null;
		}
		
		static public function removePerson(id:String):Boolean
		{
			for each(var i:uint in _people)
				if(_people[i]._id == id)
				{
					_people.splice(i,1);
					return true;
				}
			return false;
		}
		
		/*** CITY ***/
		static public function addMap(m:Map):void
		{
			_maps[m._id] = m;
		}
		
		static public function getMap(id:String):Map
		{
			return _maps[id];
		}
		
		static public function removeMap(m:Map):void
		{
			
		}
		
	}
}